﻿define([
    'jquery',
    'loglevel',
    '../../../src/js/index',
    '../config/test_cl',
    '../config/test_data.json',
    '../config/test_dsd.json'
], function ($, log, DataEditor, TestCL, TestData, TestDSD) {
        //var config = {};
        var config = {
            lang: 'FR',
            D3SConnector: { getMetaAndDataUrl: "http://faostat3.fao.org:7799/v2/msd/resources" }
        };

        console.clear();
        log.setLevel('silent');

        //Bootstrap
        require('bootstrap/dist/css/bootstrap.css');
        require("../../../node_modules/eonasdan-bootstrap-datetimepicker/build/css/bootstrap-datetimepicker.min.css");

        var callB = null;
        DataEditor.init("#standard", config, callB);
        DataEditor.isEditable(false);
        DataEditor.setColumns(TestDSD, TestCL, function() {
            DataEditor.setData(TestData);
        });

        DataEditor.on("data:loaded", function(){
            console.log(' data loaded ');
        });

        $('#btnDestroy').click(function(){
            DataEditor.destroy();
        });

        $('#btnInitAgain').click(function(){
            DataEditor.init("#standard", config, callB);
            DataEditor.isEditable(false);
            DataEditor.setColumns(TestDSD, TestCL, function() {
                DataEditor.setData(TestData, 50);
            });
        });

        $('#btnLoaded').click(function(){
            if (DataEditor.getStatus() === 'loading' ) {
                DataEditor.setStatus('loaded');
            } else {
                DataEditor.setStatus('loading');
            }
        });

        $('#btnGetData').click(function () {
            var data = DataEditor.getData();
            var cols = DataEditor.getColumnsWithDistincts();

            $('#DataOutput').html("<h3>Data</h3><code>"+data+"</code><h3>Columns</h3><code>"+JSON.stringify(cols)+"</code>")
        });

        $('#btnToggleEdit').click(function() {
            var value = DataEditor.isEditable();
            DataEditor.isEditable(!value);
        });

        $('#btnCSVLoading').click(function(){
            DataEditor.setStatus('loading');
            var one = [{"dataType":"year","key":true,"id":"DIMENSION0","title":{"FR":"y","EN":"1"},"subject":"time"},{"dataType":"code","key":true,"id":"DIMENSION1","title":{"FR":"i","EN":"2"},"domain":{"codes":[{"idCodeList":"CountrySTAT_Indicators"}]},"subject":"indicator"},{"dataType":"code","key":true,"id":"DIMENSION2","title":{"EN":"3"},"domain":{"codes":[{"idCodeList":"CountrySTAT_Residence"}]},"subject":"residence"},{"dataType":"number","key":false,"id":"VALUE0","title":{"FR":"v","EN":"4"},"subject":"value"},{"dataType":"code","key":false,"id":"OTHER0","title":{"FR":"u","EN":"5"},"domain":{"codes":[{"idCodeList":"CountrySTAT_UM"}]},"subject":"um"},{"dataType":"code","key":false,"id":"OTHER1","title":{"FR":"f","EN":"6"},"domain":{"codes":[{"idCodeList":"Flag"}]},"subject":"flag"}];
            var two = {"CountrySTAT_Indicators":{"metadata":{"title":{"EN":"CountrySTAT Indicators"},"dsd":{"rid":"68_236"},"rid":"12_849","uid":"CountrySTAT_Indicators"},"data":[{"level":1,"leaf":false,"title":{"FR":"Production","EN":"Production"},"children":[{"level":2,"leaf":true,"title":{"FR":"Superficie ensemencée","EN":"Area sown"},"code":"0105","rid":"64_94765"},{"level":2,"leaf":true,"title":{"FR":"Quantité des semences","EN":"Seeds quantity"},"code":"0106","rid":"64_94766"},{"level":2,"leaf":true,"title":{"FR":"Rendement","EN":"Yield"},"code":"0103","rid":"64_94767"},{"level":2,"leaf":true,"title":{"FR":"Superficie récoltée","EN":"Area Harvested"},"code":"0104","rid":"64_94768"},{"level":2,"leaf":true,"title":{"FR":"Quantité produite","EN":"Production quantity"},"code":"0101","rid":"64_94770"},{"level":2,"leaf":true,"title":{"FR":"Valeur de la production","EN":"Production value"},"code":"0102","rid":"64_94772"}],"code":"01","rid":"64_94762"},{"level":1,"leaf":false,"title":{"FR":"Disponibilité alimentaire","EN":"Food availability"},"children":[{"level":2,"leaf":true,"title":{"FR":"Disponibilité de protéines en quantité","EN":"Protein quantity"},"code":"0201","rid":"64_94775"},{"level":2,"leaf":true,"title":{"FR":"Disponibilité alimentaire en quantité","EN":"Food supply quantity"},"code":"0203","rid":"64_106553"},{"level":2,"leaf":true,"title":{"FR":"Disponibilité de lipides en quantité","EN":"Lipid quantity"},"code":"0202","rid":"64_94777"}],"code":"02","rid":"64_94776"},{"level":1,"leaf":false,"title":{"FR":"Commerce","EN":"Trade"},"children":[{"level":2,"leaf":true,"title":{"FR":"Quantités des importations","EN":"Import quantity"},"code":"0303","rid":"64_94778"},{"level":2,"leaf":true,"title":{"FR":"Valeur des importations","EN":"Import value"},"code":"0304","rid":"64_94780"},{"level":2,"leaf":true,"title":{"FR":"Quantités des exportations","EN":"Export quantity"},"code":"0301","rid":"64_94781"},{"level":2,"leaf":true,"title":{"FR":"Valeur des exportations","EN":"Export value"},"code":"0302","rid":"64_94782"},{"level":2,"leaf":true,"title":{"FR":"Quantité des stocks des produits disponibles","EN":"Inventory of available products"},"code":"0310","rid":"64_149841"},{"level":2,"leaf":true,"title":{"FR":"Taux de consommation","EN":"Consumption rate"},"code":"0311","rid":"64_149860"},{"level":2,"leaf":true,"title":{"FR":"Valeur des réimportations","EN":"Re-import value"},"code":"0309","rid":"64_106809"},{"level":2,"leaf":true,"title":{"FR":"Valeur des réexportations","EN":"Re-export value"},"code":"0307","rid":"64_94783"},{"level":2,"leaf":true,"title":{"FR":"Quantités des réimportations","EN":"Re-import quantity"},"code":"0308","rid":"64_106810"},{"level":2,"leaf":true,"title":{"FR":"Quantité de consommation","EN":"Consumption quantity"},"code":"0305","rid":"64_94785"},{"level":2,"leaf":true,"title":{"FR":"Quantités des réexportations","EN":"Re-export quantity"},"code":"0306","rid":"64_94786"}],"code":"03","rid":"64_94779"},{"level":1,"leaf":false,"title":{"FR":"Machines","EN":"Machinery"},"children":[{"level":2,"leaf":true,"title":{"FR":"Machines en service","EN":"Machines in service"},"code":"0401","rid":"64_94787"}],"code":"04","rid":"64_94788"},{"level":1,"leaf":false,"title":{"FR":"Population","EN":"Population"},"children":[{"level":2,"leaf":true,"title":{"FR":"Taux de croissance de la population","EN":"Population growth rate"},"code":"0504","rid":"64_94792"},{"level":2,"leaf":true,"title":{"FR":"Densité totale de la population","EN":"Population total density"},"code":"0503","rid":"64_94793"},{"level":2,"leaf":true,"title":{"FR":"Taux de malnutrition chronique","EN":"Chronic malnutrition rate"},"code":"0506","rid":"64_149862"},{"level":2,"leaf":true,"title":{"FR":"Espérance de vie","EN":"Life expectancy"},"code":"0505","rid":"64_116294"},{"level":2,"leaf":true,"title":{"FR":"Population économiquement active","EN":"Economically active population"},"code":"0502","rid":"64_94800"},{"level":2,"leaf":true,"title":{"FR":"Population","EN":"Population"},"code":"0501","rid":"64_94801"}],"code":"05","rid":"64_94790"},{"level":1,"leaf":false,"title":{"FR":"Prix","EN":"Prices"},"children":[{"level":2,"leaf":true,"title":{"FR":"Prix à la consommation","EN":"Consumer price"},"code":"0601","rid":"64_94805"},{"level":2,"leaf":true,"title":{"FR":"Taux officiel","EN":"Official rate"},"code":"0604","rid":"64_94807"},{"level":2,"leaf":true,"title":{"FR":"Prix de gros","EN":"Wholesale price"},"code":"0605","rid":"64_106811"},{"level":2,"leaf":true,"title":{"FR":"Prix au producteur","EN":"Producer price"},"code":"0602","rid":"64_94809"},{"level":2,"leaf":true,"title":{"FR":"Taux commercial","EN":"Commercial rate"},"code":"0603","rid":"64_94810"},{"level":2,"leaf":true,"title":{"FR":"Prix maximum","EN":"Highest price"},"code":"0608","rid":"64_149842"},{"level":2,"leaf":true,"title":{"FR":"Prix annuel moyen","EN":"Annual average price"},"code":"0606","rid":"64_106812"},{"level":2,"leaf":true,"title":{"FR":"Prix minimum","EN":"Lowest price"},"code":"0607","rid":"64_149843"}],"code":"06","rid":"64_94804"},{"level":1,"leaf":false,"title":{"FR":"Valeur ajoutée","EN":"Value added"},"children":[{"level":2,"leaf":true,"title":{"FR":"Prix courants","EN":"Current prices"},"code":"0702","rid":"64_94819"},{"level":2,"leaf":true,"title":{"FR":"Prix constants","EN":"Constant prices"},"code":"0701","rid":"64_94820"}],"code":"07","rid":"64_94816"},{"level":1,"leaf":false,"title":{"FR":"Terres","EN":"Land use"},"children":[{"level":2,"leaf":true,"title":{"FR":"Terres arables","EN":"Arable land"},"code":"0803","rid":"64_94821"},{"level":2,"leaf":true,"title":{"FR":"Superficie des cultures permanentes","EN":"Permanent crops area"},"code":"0812","rid":"64_116295"},{"level":2,"leaf":true,"title":{"FR":"Superficie agricole","EN":"Agricultural area"},"code":"0802","rid":"64_94823"},{"level":2,"leaf":true,"title":{"FR":"Autres terres boisées","EN":"Other wooded land"},"code":"0811","rid":"64_116296"},{"level":2,"leaf":true,"title":{"FR":"Superficie des cultures temporaires","EN":"Temporary crops area"},"code":"0814","rid":"64_116297"},{"level":2,"leaf":true,"title":{"FR":"Superficie administrative","EN":"Administrative Area"},"code":"0801","rid":"64_94824"},{"level":2,"leaf":true,"title":{"FR":"Prairies et pâturages permanents","EN":"Permanent meadows and pastures"},"code":"0813","rid":"64_116298"},{"level":2,"leaf":true,"title":{"FR":"Superficie forestière","EN":"Forest area"},"code":"0807","rid":"64_116299"},{"level":2,"leaf":true,"title":{"FR":"Prairies et pâturages temporaires","EN":"Temporary meadows and pastures"},"code":"0815","rid":"64_116300"},{"level":2,"leaf":true,"title":{"FR":"Terres en jachère","EN":"Fallow Land"},"code":"0806","rid":"64_94826"},{"level":2,"leaf":true,"title":{"FR":"Superficie équipée pour l'irrigation","EN":"Area equipped for irrigation"},"code":"0805","rid":"64_94827"},{"level":2,"leaf":true,"title":{"FR":"Terres arables et superficie des cultures permanentes","EN":"Arable land and permanent crops area"},"code":"0804","rid":"64_94828"},{"level":2,"leaf":true,"title":{"FR":"Autres terres","EN":"Other land"},"code":"0810","rid":"64_116301"},{"level":2,"leaf":true,"title":{"FR":"Eaux intérieures","EN":"Inland water"},"code":"0808","rid":"64_116302"},{"level":2,"leaf":true,"title":{"FR":"Extension des terres","EN":"Land Area"},"code":"0809","rid":"64_116303"}],"code":"08","rid":"64_94822"},{"level":1,"leaf":false,"title":{"FR":"Emploi","EN":"Employment"},"children":[{"level":2,"leaf":true,"title":{"FR":"Taux de chômage","EN":"Unemployment Rate"},"code":"0901","rid":"64_94832"},{"level":2,"leaf":true,"title":{"FR":"Taux de participation à la population active","EN":"Labour force Participation Rate"},"code":"0902","rid":"64_94833"}],"code":"09","rid":"64_94831"},{"level":1,"leaf":false,"title":{"FR":"Eau","EN":"Water"},"children":[{"level":2,"leaf":true,"title":{"FR":"Température minimale","EN":"Min temperature"},"code":"1003","rid":"64_106813"},{"level":2,"leaf":true,"title":{"FR":"Taux de desserte en Eau Potable","EN":"Drinking-water supply rate"},"code":"1004","rid":"64_149844"},{"level":2,"leaf":true,"title":{"FR":"Hauteur moyenne de précipitations","EN":"Average rainfall"},"code":"1001","rid":"64_106554"},{"level":2,"leaf":true,"title":{"FR":"Température maximale","EN":"Max temperature"},"code":"1002","rid":"64_106814"},{"level":2,"leaf":true,"title":{"FR":"Taux d'accès en Assainissement de base","EN":"Basic sanitation access rate"},"code":"1005","rid":"64_149845"}],"code":"10","rid":"64_106555"},{"level":1,"leaf":false,"title":{"FR":"Elevage","EN":"Livestock"},"children":[{"level":2,"leaf":true,"title":{"FR":"Sortie totale","EN":"Total Output"},"code":"1106","rid":"64_116304"},{"level":2,"leaf":true,"title":{"FR":"Animaux morts","EN":"Dead Animals"},"code":"1104","rid":"64_116306"},{"level":2,"leaf":true,"title":{"FR":"Entrée totale","EN":"Total Input"},"code":"1105","rid":"64_116307"},{"level":2,"leaf":false,"title":{"FR":"Animaux vivants","EN":"Live animals"},"children":[{"level":3,"leaf":true,"title":{"FR":"Animaux vivants femelles","EN":"Female Live Animals"},"code":"110201","rid":"64_149846"},{"level":3,"leaf":true,"title":{"FR":"âge de réforme","EN":"Age at disposal"},"code":"110212","rid":"64_150228"},{"level":3,"leaf":true,"title":{"FR":"Animaux vivants mâles","EN":"Male Live Animals"},"code":"110203","rid":"64_116309"},{"level":3,"leaf":true,"title":{"FR":"Animaux donnés en cadeau","EN":"Animals given as gift"},"code":"110211","rid":"64_116310"},{"level":3,"leaf":true,"title":{"FR":"Animaux en lactation","EN":"Lactating Animals"},"code":"110202","rid":"64_149847"},{"level":3,"leaf":true,"title":{"FR":"Animaux donnés en gardiennage","EN":"Animals given for caretaking"},"code":"110210","rid":"64_116311"},{"level":3,"leaf":true,"title":{"FR":"Animaux achetés","EN":"Bought Animals"},"code":"110205","rid":"64_116312"},{"level":3,"leaf":true,"title":{"FR":"Animaux nés","EN":"Born Animals"},"code":"110204","rid":"64_116313"},{"level":3,"leaf":true,"title":{"FR":"Animaux reçus en gardiennage","EN":"Animals got for caretaking"},"code":"110207","rid":"64_116314"},{"level":3,"leaf":true,"title":{"FR":"Animaux reçus en cadeau","EN":"Animals got as gift"},"code":"110206","rid":"64_116315"},{"level":3,"leaf":true,"title":{"FR":"Animaux volés","EN":"Stolen Animals"},"code":"110209","rid":"64_116316"},{"level":3,"leaf":true,"title":{"FR":"Animaux vendus","EN":"Sold Animals"},"code":"110208","rid":"64_116317"}],"code":"1102","rid":"64_116308"},{"level":2,"leaf":true,"title":{"FR":"Pondeuses","EN":"Laying hens"},"code":"1103","rid":"64_116318"},{"level":2,"leaf":true,"title":{"FR":"Animaux abattus","EN":"Slaughtered animals"},"code":"1101","rid":"64_116319"}],"code":"11","rid":"64_116305"},{"level":1,"leaf":false,"title":{"FR":"Comptes nationaux","EN":"National account"},"children":[{"level":2,"leaf":true,"title":{"FR":"PIB par habitant","EN":"GDP per capita"},"code":"1203","rid":"64_116322"},{"level":2,"leaf":true,"title":{"FR":"Croissance du PIB","EN":"GDP growth"},"code":"1202","rid":"64_116324"},{"level":2,"leaf":true,"title":{"FR":"PIB","EN":"GPD"},"code":"1201","rid":"64_116325"}],"code":"12","rid":"64_116323"},{"level":1,"leaf":false,"title":{"FR":"Pêches","EN":"Fishery"},"children":[{"level":2,"leaf":false,"title":{"FR":"Production aquacole","EN":"Aquaculture production"},"children":[{"level":3,"leaf":true,"title":{"FR":"Production aquacole en eaux saumâtres","EN":"Marine/Brackish Aquaculture production"},"code":"130103","rid":"64_116332"},{"level":3,"leaf":true,"title":{"FR":"Production aquacole en eaux intérieures","EN":"Inland Aquaculture production"},"code":"130101","rid":"64_116333"},{"level":3,"leaf":false,"title":{"FR":"Production aquacole en eaux marines","EN":"Marine Aquaculture production"},"children":[{"level":4,"leaf":true,"title":{"FR":"Production aquacole industrielle en eaux marines","EN":"Industrial Marine Aquaculture production"},"code":"13010201","rid":"64_116978"},{"level":4,"leaf":true,"title":{"FR":"Production aquacole artisanale en eaux marines","EN":"Artisanal Marine Aquaculture production"},"code":"13010202","rid":"64_116979"}],"code":"130102","rid":"64_116334"}],"code":"1301","rid":"64_116326"},{"level":2,"leaf":false,"title":{"FR":"Production des captures","EN":"Capture production"},"children":[{"level":3,"leaf":false,"title":{"FR":"Production des captures en eaux marines","EN":"Marine Capture production"},"children":[{"level":4,"leaf":true,"title":{"FR":"Production artisanale des captures en eaux marines","EN":"Artisanal Marine Capture production"},"code":"13020202","rid":"64_116976"},{"level":4,"leaf":true,"title":{"FR":"Production traditionnelle des captures en eaux marines","EN":"Traditional Marine Capture production"},"code":"13020203","rid":"64_149848"},{"level":4,"leaf":true,"title":{"FR":"Production industrielle des captures en eaux marines","EN":"Industrial Marine Capture production"},"code":"13020201","rid":"64_116977"}],"code":"130202","rid":"64_116329"},{"level":3,"leaf":true,"title":{"FR":"Production des captures en eaux saumâtres","EN":"Marine/Brackish Capture production"},"code":"130203","rid":"64_116330"},{"level":3,"leaf":true,"title":{"FR":"Production des captures en eaux intérieures","EN":"Inland Capture production"},"code":"130201","rid":"64_116331"}],"code":"1302","rid":"64_116328"}],"code":"13","rid":"64_116327"},{"level":1,"leaf":false,"title":{"FR":"Forêts","EN":"Forestry"},"children":[{"level":2,"leaf":true,"title":{"FR":"Taux de déforestation","EN":"Deforestation rate"},"code":"1405","rid":"64_149849"},{"level":2,"leaf":true,"title":{"FR":"Superficie reboisée","EN":"Reforested area"},"code":"1401","rid":"64_149851"},{"level":2,"leaf":true,"title":{"FR":"Plants mises en terre","EN":"Seedlings planted"},"code":"1402","rid":"64_149852"},{"level":2,"leaf":false,"title":{"FR":"Superficie brûlée","EN":"Burned area"},"children":[{"level":3,"leaf":true,"title":{"FR":"Prairies  brûlées","EN":"Burned meadows"},"code":"140301","rid":"64_149854"},{"level":3,"leaf":true,"title":{"FR":"Défrichements Non Autorisés","EN":"Non-authorized clearing"},"code":"140305","rid":"64_149855"},{"level":3,"leaf":true,"title":{"FR":"Défrichements Autorisés","EN":"Authorized clearing"},"code":"140304","rid":"64_149856"},{"level":3,"leaf":true,"title":{"FR":"Reboisement brûlé","EN":"Burned reafforestation"},"code":"140303","rid":"64_149857"},{"level":3,"leaf":true,"title":{"FR":"Forêts naturelles brûlées","EN":"Burned natural forests"},"code":"140302","rid":"64_149858"}],"code":"1403","rid":"64_149853"},{"level":2,"leaf":true,"title":{"FR":"Superficie déforéstée","EN":"Deforested area"},"code":"1404","rid":"64_149859"}],"code":"14","rid":"64_149850"}],"size":117},"CountrySTAT_Residence":{"metadata":{"title":{"EN":"Residence"},"dsd":{"rid":"68_248"},"rid":"12_909","uid":"CountrySTAT_Residence"},"data":[{"level":1,"leaf":true,"title":{"FR":"Rural(e)","EN":"Rural"},"code":"7001","rid":"64_116464"},{"level":1,"leaf":true,"title":{"FR":"Urbain(e)","EN":"Urban"},"code":"7002","rid":"64_116465"},{"level":1,"leaf":true,"title":{"FR":"Non spécifié","EN":"Unspecified"},"code":"7003","rid":"64_116466"},{"level":1,"leaf":true,"title":{"FR":"Total","EN":"Total"},"code":"7004","rid":"64_116467"}],"size":4},"CountrySTAT_UM":{"metadata":{"title":{"EN":"Units of measurement"},"dsd":{"rid":"68_237"},"rid":"12_854","uid":"CountrySTAT_UM"},"data":[{"level":1,"leaf":true,"title":{"FR":"tête","EN":"head"},"code":"0101","rid":"64_94903"},{"level":1,"leaf":true,"title":{"FR":"Ha","EN":"Ha"},"code":"0102","rid":"64_94904"},{"level":1,"leaf":true,"title":{"FR":"Tonnes","EN":"ton"},"code":"0103","rid":"64_94905"},{"level":1,"leaf":true,"title":{"FR":"US$","EN":"US$"},"code":"0104","rid":"64_94906"},{"level":1,"leaf":true,"title":{"FR":"Monnaie locale","EN":"local currency"},"code":"0105","rid":"64_94907"},{"level":1,"leaf":true,"title":{"FR":"Indice","EN":"index"},"code":"0106","rid":"64_94908"},{"level":1,"leaf":true,"title":{"FR":"Miliers","EN":"thousands"},"code":"0107","rid":"64_94909"},{"level":1,"leaf":true,"title":{"FR":"Unité","EN":"units"},"code":"0108","rid":"64_94910"},{"level":1,"leaf":true,"title":{"FR":"Mètre cube","EN":"cubic meter"},"code":"0109","rid":"64_94911"},{"level":1,"leaf":true,"title":{"FR":"Kg/personne/an","EN":"kg/capita/yr"},"code":"0110","rid":"64_94912"},{"level":1,"leaf":true,"title":{"FR":"Kcal/personne/jour","EN":"Kcal/capita/day"},"code":"0111","rid":"64_94913"},{"level":1,"leaf":true,"title":{"FR":"g/personne/jour","EN":"g/capita/day"},"code":"0112","rid":"64_94914"},{"level":1,"leaf":true,"title":{"FR":"1000 US$","EN":"1000 US$"},"code":"0113","rid":"64_94915"},{"level":1,"leaf":true,"title":{"FR":"Litre","EN":"Liter"},"code":"0114","rid":"64_94916"},{"level":1,"leaf":true,"title":{"FR":"Mètre cube/seconde","EN":"Cubic meter/second"},"code":"0115","rid":"64_94917"},{"level":1,"leaf":true,"title":{"FR":"Centimetre","EN":"Centimeter"},"code":"0116","rid":"64_94918"},{"level":1,"leaf":true,"title":{"FR":"Monnaie locale par tonne","EN":"Local currency by ton"},"code":"0117","rid":"64_94919"},{"level":1,"leaf":true,"title":{"FR":"Degré celsius","EN":"Degree Celsius"},"code":"0118","rid":"64_94920"},{"level":1,"leaf":true,"title":{"FR":"Pourcentage","EN":"Percentage"},"code":"0119","rid":"64_94921"},{"level":1,"leaf":true,"title":{"FR":"Tonne métrique ou mètre cube","EN":"MT or Cubic meter"},"code":"0120","rid":"64_94922"},{"level":1,"leaf":true,"title":{"FR":"Monnaie locale/hectare","EN":"Local currency/hectar"},"code":"0121","rid":"64_94923"},{"level":1,"leaf":true,"title":{"FR":"Monnaie locale/mètre cube","EN":"Local currency/1 cubic meter"},"code":"0122","rid":"64_94924"},{"level":1,"leaf":true,"title":{"FR":"Kg/Ha","EN":"Kg/Ha"},"code":"0123","rid":"64_94925"},{"level":1,"leaf":true,"title":{"FR":"Monnaie locale/Kg","EN":"Local currency/Kg"},"code":"0124","rid":"64_94926"},{"level":1,"leaf":true,"title":{"FR":"1000 Ha","EN":"1000 Ha"},"code":"0125","rid":"64_94927"},{"level":1,"leaf":true,"title":{"FR":"mm","EN":"mm"},"code":"0126","rid":"64_94928"},{"level":1,"leaf":true,"title":{"FR":"Mètre carré","EN":"Square meter"},"code":"0127","rid":"64_94929"},{"level":1,"leaf":true,"title":{"FR":"Famille","EN":"Household"},"code":"0128","rid":"64_106815"},{"level":1,"leaf":true,"title":{"FR":"Tonne/Ha","EN":"Ton/Ha"},"code":"0129","rid":"64_106816"},{"level":1,"leaf":true,"title":{"FR":"Kilomètres carrés","EN":"Km2"},"code":"0130","rid":"64_106817"},{"level":1,"leaf":true,"title":{"FR":"Années","EN":"Years"},"code":"0131","rid":"64_116335"},{"level":1,"leaf":true,"title":{"FR":"Monnaie locale/dollar","EN":"Local currency/dollar"},"code":"0132","rid":"64_116336"},{"level":1,"leaf":true,"title":{"FR":"1000 Monnaie locale","EN":"1000 Local currency"},"code":"0133","rid":"64_116974"},{"level":1,"leaf":true,"title":{"FR":"Kg","EN":"Kg"},"code":"0134","rid":"64_116975"},{"level":1,"leaf":true,"title":{"FR":"Monnaie locale/litre","EN":"Local currency/Litre"},"code":"0135","rid":"64_149861"},{"level":1,"leaf":true,"title":{"FR":"Jours","EN":"Days"},"code":"0136","rid":"64_149863"},{"level":1,"leaf":true,"title":{"FR":"Litre par traite par tête","EN":"Liter/milking/head"},"code":"0137","rid":"64_149864"},{"level":1,"leaf":true,"title":{"FR":"Mois","EN":"Months"},"code":"0138","rid":"64_149865"}],"size":38},"Flag":{"metadata":{"title":{"EN":"Flag"},"dsd":{"rid":"68_243"},"rid":"12_1512","uid":"Flag"},"data":[{"level":1,"leaf":true,"title":{"FR":"calculé(e)","EN":"calculated"},"code":"c","rid":"64_150193"},{"level":1,"leaf":true,"title":{"FR":"estimé(e)","EN":"estimated"},"code":"e","rid":"64_150194"}],"size":2}} ;
            //var thr = [["20161025","2"],["20161025","3"],["20161025","4"],["20161025","1000"]];
            DataEditor.setColumns(one, two, null);



            // colonne maggiori ma corretto
            var csv = 'Année,Indicateur (code),Residence (code),Value,Flag,um (code),xxx\r\n2003,0501,7001,1294,,0107\r\n2003,0501,7002,1896,,0107,\r\n2003,0501,7002,1896,,0107,\r\n2004,0501,7002,1968,,0107,\r\n2004,0501,7001,1323,,0107,\r\n2004,0501,7001,1323,,0107,\r\n2004,0501,7001,1323,,0107,';

            //console.log(JSON.stringify(csv));

            //var csv = 'Année,Indicateur (code),Residence (code),Value,Flag,um (code)\r\n2003,0501,7001,1294,,0107\r\n2003,0501,7002,1896,,0107\r\n2003,0501,7002,1896,,0107\r\n2004,0501,7002,1968,,0107\r\n2004,0501,7001,1323,,0107\r\n2004,0501,7001,1323,,0107\r\n2004,0501,7001,1323,,0107';
            DataEditor.csvLoaded(csv,{},';')

        });

    }
);
